#include <stdio.h>
#include <malloc.h>
#include <time.h>

#define ARRAY_SIZE 1000
#define ITERATIONS 100000

int multi_array[ARRAY_SIZE][ARRAY_SIZE];
int **multi_ptr;

/* run "benchmark" with true multidimensional arrays*/
double test_true_multidimensional()
{
	clock_t start, end;

	start = clock();
	for (int i = 0; i < ARRAY_SIZE; ++i)
	{
		for (int j = 0; j < ARRAY_SIZE; ++j)
		{
			multi_array[i][j] = (i + j);
		}
	}
	for (int i = 0; i < ARRAY_SIZE; ++i)
	{
		for (int j = 0; j < ARRAY_SIZE; ++j)
		{
			multi_array[i][j] = (i - j);
		}
	}
	end = clock();

	return (end - start)/(double)1000; /* 1000 statt CLOCKS_PER_CYCLE um ms zu erhalten*/
}

void init_ptr()
{
	multi_ptr = malloc(ARRAY_SIZE * sizeof(*multi_ptr));

	for (int i = 0; i < ARRAY_SIZE; ++i)
	{
		multi_ptr[i] = malloc(ARRAY_SIZE * sizeof(**multi_ptr));
	}
}

void free_ptr()
{
	for (int i = 0; i < ARRAY_SIZE; ++i)
	{
		free(multi_ptr[i]);
	}
	free(multi_ptr);
}

/* run benchmark with pointer->pointer "arrays" */
double test_ptr_to_ptr()
{
	clock_t start, end;

	start = clock();
	for (int i = 0; i < ARRAY_SIZE; ++i)
	{
		for (int j = 0; j < ARRAY_SIZE; ++j)
		{
			multi_ptr[i][j] = (i + j);
		}
	}
	for (int i = 0; i < ARRAY_SIZE; ++i)
	{
		for (int j = 0; j < ARRAY_SIZE; ++j)
		{
			multi_ptr[i][j] = (i - j);
		}
	}
	end = clock();

	return (end - start)/(double)1000; /* 1000 statt CLOCKS_PER_CYCLE um ms zu erhalten*/
}

int main(int argc, char *argv[])
{
	double total_array = 0, total_ptr = 0;
	double average_array = 0, average_ptr = 0;

	printf("Starting benchmark with ARRAY_SIZE = %d and %d iterations\n\n", ARRAY_SIZE, ITERATIONS);

	printf("Start measureing [][]..\n");
	for (int i = 0; i < ITERATIONS; ++i)
	{
		total_array += test_true_multidimensional();
	}
	average_array = total_array / (double)ITERATIONS;
	printf("Finished measureing [][]..\n\n");

	init_ptr();
	printf("Start measureing **..\n");
	for (int i = 0; i < ITERATIONS; ++i)
	{
		total_ptr += test_ptr_to_ptr();
	}
	average_ptr = total_ptr / (double)ITERATIONS;
	printf("Finished measureing **..\n\n");
	free_ptr();

	/* printf("Total [][]: %d ms\n", total_array); */
	printf("Average [][]: %f ms\n", average_array);
	/* printf("Total **: %d ms\n", total_ptr); */
	printf("Average **: %f ms\n", average_ptr);

	return 1;
}